/*
    ES Knot
    Copyright 2005 - 2008 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#ifndef __KNOT_H
#define __KNOT_H

#include "knotinterface.h"

#include <GLES/gl.h>

#ifdef USE_OES_2
// If using this header (in a mixed ES1/2 environment), don't have OES_2 defined here
// when including glmath.h
#undef USE_OES_2
#endif

#include "glmath.h"

#ifdef __APPLE__
#define USE_HARDWARE
#endif

#ifdef USE_HARDWARE
#define RINGS 150
#define POINTS 16
#else
#define RINGS 70
#define POINTS 8
#endif
#define SINETABLESIZE 512
#define SINETABLEMASK 0x01FF
#define SINETABLEMASKSIZE 9

class RadialBlur;
class MotionBlur;

class Knot : public KnotInterface {
public:
	Knot(int w, int h);
	~Knot();
	void draw(unsigned int msec);
	void reinit();
	void clearContext();
	void setSize(int w, int h);

	void setTexMode(TexMode mode);
	void setUpdateVertices(bool update);
	void setMotionBlur(bool blur);
	void setRadialBlur(bool blur);
	void setRadialGlow(bool glow);

	void startMorph();

	void enableRotate(bool rotate);
	void enableGravity(bool gravity);
	void rotate(int x, int y);
	
	void setGravity(float x, float y, float z);

	bool getMatrixSupported();

private:
	TYPE vertices[3*(RINGS+1)*(POINTS+1)];
	TYPE outlineVertices[3*(RINGS+1)*(POINTS+1)];
	TYPE normals[3*(RINGS+1)*(POINTS+1)];
	TYPE texcoords[2*(RINGS+1)*(POINTS+1)];
	TYPE celcoords[2*(RINGS+1)*(POINTS+1)];
	GLushort indices[6*RINGS*POINTS];
	TYPE ringvertices[4*POINTS];
	TYPE ringcoords23[16*RINGS];
	TYPE ringcoords25[16*RINGS];
	TYPE *ringcoords;
	TYPE ringcoordstemp[16*RINGS];

	static void polar(float pos, TYPE *vect, int P, int Q);
	void precalc();
	void initProjection();
	void genTexture();
	void genCelTexture();
	void updateCelCoords(TYPE *verts, TYPE *norms);
	void updateOutlineVertices(TYPE *outline, const TYPE *verts, const TYPE *norms);
	bool getModelview(TYPE* matrix);
	void getLightInverse(TYPE* light);
	void calcDynVertices();
	void calcDynVerticesNormals(TYPE *verts, TYPE *norms, const TYPE *coords);
	void makeRingCoords(TYPE *coords, int P, int Q);

	TYPE fastSin(TYPE x);
	void setDrawState();
	void morphMatrices(TYPE *out, const TYPE *first, const TYPE *second, TYPE pos);
	
	void rotateGravity();

	void reinit(TexMode mode);
	int width, height;

	unsigned int lasttime;
	bool morphing;
	unsigned int morphStart;

	GLuint texture;
	GLuint celTexture;
	TexMode texMode;
	TYPE texOffsetX, texOffsetY;

	bool updateBumps;
	TYPE bumpOffset;

	TYPE sinTable[SINETABLESIZE];

	RadialBlur* radialBlur;
	bool doRadialBlur;

	MotionBlur* motionBlur;
	bool doMotionBlur;

	GLbitfield (*ext_glQueryMatrixxOES)(GLfixed mantissa[16], GLint exponent[16]);
	void (*ext_glGetTypev)(GLenum, TYPE*);

	bool rotating;
	
	bool gravityEnabled;
	bool gravitySet;
	TYPE curGravity[3];
	TYPE targetGravity[3];

	TYPE complementVect[3];
	TYPE extraRotMatrix[16];
	bool extraRotInited;
};

#endif
