/*
    ES Knot
    Copyright 2005, 2006 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#ifdef UIQ3
#include <qikon.hrh>
#include <uikon.hrh>
#else
#include <avkon.hrh>
#endif
#include <flogger.h>

#include "knot.pan"
#include "knotappui.h"
#include "knotappview.h"
#include "knot.hrh"
#ifdef USE_OES_2
#include "knot2.h"
#else
#include "knot.h"
#endif
#include <knot.rsg>

void CKnotAppUi::ConstructL() {
#ifdef UIQ3
	CQikAppUi::ConstructL();
	iAppView = CKnotAppView::NewLC(*this);
	AddViewL(*iAppView);
	CleanupStack::Pop(iAppView);
#else
	BaseConstructL();
	iAppView = CKnotAppView::NewL(ClientRect());
	AddToStackL(iAppView);
#endif
}

CKnotAppUi::CKnotAppUi() {
}

CKnotAppUi::~CKnotAppUi() {
#ifndef UIQ3
	if (iAppView) {
		RemoveFromStack(iAppView);
		delete iAppView;
		iAppView = NULL;
	}
#endif
}

void CKnotAppUi::HandleCommandL(TInt aCommand) {
	CKnotAppView* appView = static_cast<CKnotAppView*>(iAppView);
	switch(aCommand) {
	case EEikCmdExit:
#ifndef UIQ3
	case EAknSoftkeyExit:
	case EAknSoftkeyBack:
#endif
		Exit();
		break;

	case EKnotUntextured:
		appView->iKnot->setTexMode(Knot::UNTEXTURED);
		break;

	case EKnotPlainTexture:
		appView->iKnot->setTexMode(Knot::PLAINTEX);
		break;

	case EKnotTexSpecular:
		appView->iKnot->setTexMode(Knot::TEXSPEC);
		break;

	case EKnotCelshaded:
		appView->iKnot->setTexMode(Knot::CELSHADED);
		break;

	case EKnotDrawEGLWindow:
		appView->SetDrawMode(CKnotAppView::EEGLWindow);
		break;
	case EKnotDrawPixmap:
		appView->SetDrawMode(CKnotAppView::EPixmap, EFalse);
		break;
	case EKnotDrawPixmapDSA:
		appView->SetDrawMode(CKnotAppView::EPixmap, ETrue);
		break;
	case EKnotDrawPbuffer:
		appView->SetDrawMode(CKnotAppView::EPbuffer, EFalse);
		break;
	case EKnotDrawPbufferDSA:
		appView->SetDrawMode(CKnotAppView::EPbuffer, ETrue);
		break;

	case EKnotWinFullscreen:
	case EKnotWinWindowed:
		appView->SetWindowMode(aCommand == EKnotWinFullscreen ? CKnotAppView::EFullscreen : CKnotAppView::EWindowed);
		break;

	case EKnotVertexUpdateOn:
	case EKnotVertexUpdateOff:
		appView->iKnot->setUpdateVertices(aCommand == EKnotVertexUpdateOn);
		break;

	case EKnotMotionBlurOn:
	case EKnotMotionBlurOff:
		appView->iKnot->setMotionBlur(aCommand == EKnotMotionBlurOn);
		break;

	case EKnotRadialBlurGlowOn:
	case EKnotRadialBlurOn:
	case EKnotRadialBlurOff:
		appView->iKnot->setRadialBlur(aCommand == EKnotRadialBlurOn || aCommand == EKnotRadialBlurGlowOn);
		appView->iKnot->setRadialGlow(aCommand == EKnotRadialBlurGlowOn);
		break;

	case EKnotMorph:
		appView->iKnot->startMorph();
		break;

	default:
		Panic(EKnotUi);
		break;
	}
}



