/*
    ES Knot
    Copyright 2008 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#include <windows.h>
#include <aygshell.h>
#include <GLES/egl.h>
#include "knot.h"
#include "resource.h"
#include <tchar.h>

EGLDisplay display;
EGLSurface surface;
EGLContext context;
EGLConfig config;
int curWidth, curHeight;

bool buttonDown = false;
int lastX, lastY;
int distance;
Knot* knot;
int lastTime;
BOOL active = TRUE;

HINSTANCE hInst;
HWND menuBar = NULL;

Knot::TexMode texMode = Knot::TEXSPEC;
bool vertexUpdate = true;
bool motionBlur = false;
bool radialBlur = false;
bool glow = false;

void updateMenuChecks() {
	HMENU menu;

	menu = (HMENU)SendMessage(menuBar, SHCMBM_GETSUBMENU, (WPARAM)0, (LPARAM)IDM_MENU);

	CheckMenuItem(menu, IDM_UNTEX,     (texMode == Knot::UNTEXTURED) ? MF_CHECKED : MF_UNCHECKED);
	CheckMenuItem(menu, IDM_TEX,       (texMode == Knot::PLAINTEX)   ? MF_CHECKED : MF_UNCHECKED);
	CheckMenuItem(menu, IDM_TEXSPEC,   (texMode == Knot::TEXSPEC)    ? MF_CHECKED : MF_UNCHECKED);
	CheckMenuItem(menu, IDM_CELSHADED, (texMode == Knot::CELSHADED)  ? MF_CHECKED : MF_UNCHECKED);

	CheckMenuItem(menu, IDM_VERTEX_UPDATE_ON,  (vertexUpdate)  ? MF_CHECKED : MF_UNCHECKED);
	CheckMenuItem(menu, IDM_VERTEX_UPDATE_OFF, (!vertexUpdate) ? MF_CHECKED : MF_UNCHECKED);

	CheckMenuItem(menu, IDM_BLUR_ON,  (motionBlur)  ? MF_CHECKED : MF_UNCHECKED);
	CheckMenuItem(menu, IDM_BLUR_OFF, (!motionBlur) ? MF_CHECKED : MF_UNCHECKED);

	CheckMenuItem(menu, IDM_RADIAL_GLOW, (glow)                 ? MF_CHECKED : MF_UNCHECKED);
	CheckMenuItem(menu, IDM_RADIAL_ON,   (radialBlur && !glow)  ? MF_CHECKED : MF_UNCHECKED);
	CheckMenuItem(menu, IDM_RADIAL_OFF,  (!radialBlur && !glow) ? MF_CHECKED : MF_UNCHECKED);
}

LRESULT CALLBACK windowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {

	switch (uMsg) {
	case WM_CREATE:
		{
		SHMENUBARINFO shmbi;

		ZeroMemory(&shmbi, sizeof(shmbi));
		shmbi.cbSize = sizeof(shmbi);
		shmbi.hwndParent = hWnd;
		shmbi.nToolBarId = IDR_MENUBAR;
		shmbi.hInstRes = hInst;
		shmbi.dwFlags = SHCMBF_HIDESIPBUTTON;

		if (SHCreateMenuBar(&shmbi)) {
			menuBar = shmbi.hwndMB;
			if (NULL != shmbi.hwndMB) {
				// Get the main menu and store it
				//g_hMainMenu = (HMENU)SendMessage(shmbi.hwndMB, SHCMBM_GETSUBMENU, 0, IDM_MENU);
			}
		}

		break;
		}

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDM_MORPH:
			knot->startMorph();
			break;
		case IDM_UNTEX:
			texMode = Knot::UNTEXTURED;
			knot->setTexMode(texMode);
			break;
		case IDM_TEX:
			texMode = Knot::PLAINTEX;
			knot->setTexMode(texMode);
			break;
		case IDM_TEXSPEC:
			texMode = Knot::TEXSPEC;
			knot->setTexMode(texMode);
			break;
		case IDM_CELSHADED:
			texMode = Knot::CELSHADED;
			knot->setTexMode(texMode);
			break;
		case IDM_VERTEX_UPDATE_ON:
			vertexUpdate = true;
			knot->setUpdateVertices(vertexUpdate);
			break;
		case IDM_VERTEX_UPDATE_OFF:
			vertexUpdate = false;
			knot->setUpdateVertices(vertexUpdate);
			break;
		case IDM_BLUR_ON:
			motionBlur = true;
			knot->setMotionBlur(motionBlur);
			break;
		case IDM_BLUR_OFF:
			motionBlur = false;
			knot->setMotionBlur(motionBlur);
			break;
		case IDM_RADIAL_GLOW:
			glow = true;
			radialBlur = true;
			knot->setRadialBlur(radialBlur);
			knot->setRadialGlow(glow);
			break;
		case IDM_RADIAL_ON:
			glow = false;
			radialBlur = true;
			knot->setRadialBlur(radialBlur);
			knot->setRadialGlow(glow);
			break;
		case IDM_RADIAL_OFF:
			glow = false;
			radialBlur = false;
			knot->setRadialBlur(radialBlur);
			knot->setRadialGlow(glow);
			break;
		case IDM_EXIT:
			PostQuitMessage(0);
			break;
		}
		updateMenuChecks();
	break;

	case WM_PAINT:
		if (knot) {
			knot->draw(lastTime);
			eglSwapBuffers(display, surface);
		}

		/*
		{
		PAINTSTRUCT ps;

		HDC dc = BeginPaint(hWnd, &ps);
		ExtTextOut(dc, 0, 0, 0, NULL, L"Hello world!", 10, NULL);
		EndPaint(hWnd, &ps);
		}
		*/
		break;

	case WM_CLOSE:
	case WM_DESTROY:
		PostQuitMessage(0);
		break;

	case WM_LBUTTONDOWN:
		buttonDown = true;
		knot->enableRotate(false);
		lastX = LOWORD(lParam);
		lastY = HIWORD(lParam);
		distance = 0;
		break;
	case WM_MOUSEMOVE:
		{
			int newX = LOWORD(lParam);
			int newY = HIWORD(lParam);
			int diffX = newX - lastX;
			int diffY = newY - lastY;
			lastX = newX;
			lastY = newY;
			distance += abs(diffX) + abs(diffY);
			knot->rotate(diffX, diffY);
			break;
		}
	case WM_LBUTTONUP:
		knot->enableRotate(true);
		if (distance < 5)
			knot->startMorph();
		buttonDown = false;
		break;
	case WM_ACTIVATE:
		active = (BOOL) (LOWORD(wParam));
		break;
	case WM_SIZE:
		{
			RECT rect;
			GetClientRect(hWnd, &rect);
			int newWidth = rect.right - rect.left; 
			int newHeight = rect.bottom - rect.top;
			if (display != EGL_NO_DISPLAY && (newWidth != curWidth || newHeight != curHeight)) {
				curWidth = newWidth;
				curHeight = newHeight;
				eglDestroySurface(display, surface);

				surface = eglCreateWindowSurface(display, config, hWnd, NULL);
				if (!surface) {
					return 1;
				}
				if (eglMakeCurrent(display, surface, surface, context) == EGL_FALSE) {
					return 1;
				}

				knot->setSize(curWidth, curHeight);
				knot->reinit();
			}
			break;
		}

	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow) {
	hInst = hInstance;

#define WINDOW_TITLE L"Torus Knot"
#define CLASS_NAME L"MyWndClass"

	HWND wnd = FindWindow(CLASS_NAME, WINDOW_TITLE);
	if (wnd) {
		SetForegroundWindow((HWND)((UINT_PTR)wnd | 0x01));
		return 0;
	}

	WNDCLASS wc;

	ZeroMemory(&wc, sizeof(wc));
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = windowProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
//	wc.hbrBackground = CreateSolidBrush(RGB(0, 0, 0));
	wc.hbrBackground = (HBRUSH) COLOR_WINDOW;
	wc.lpszMenuName = NULL;
	wc.lpszClassName = CLASS_NAME;
	RegisterClass(&wc);

	wnd = CreateWindow(CLASS_NAME, WINDOW_TITLE, WS_VISIBLE, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);

//	SHFullScreen(hWnd, SHFS_HIDESIPBUTTON /*| SHFS_HIDESTARTICON | SHFS_HIDETASKBAR */);
	TCHAR platform[20];
	bool ppc = false;
	if (SystemParametersInfo(SPI_GETPLATFORMTYPE, sizeof(platform), platform, 0) != 0) {
		if (!_tcscmp(platform, _T("PocketPC")))
			ppc = true;
	}
	if (menuBar && ppc) {
		RECT contentWindow, menuWindow;
		GetWindowRect(wnd, &contentWindow);
		GetWindowRect(menuBar, &menuWindow);
		contentWindow.bottom -= menuWindow.bottom - menuWindow.top;
		MoveWindow(wnd, contentWindow.left, contentWindow.top, contentWindow.right - contentWindow.left, contentWindow.bottom - contentWindow.top, TRUE);
	}
	updateMenuChecks();

	display = eglGetDisplay(EGL_DEFAULT_DISPLAY);
	eglInitialize(display, NULL, NULL);
/*
	FILE* log = fopen("\\gles.txt", "w");
	fprintf(log, "vendor: %s\n", glGetString(GL_VENDOR));
	fprintf(log, "renderer: %s\n", glGetString(GL_RENDERER));
	fprintf(log, "version: %s\n", glGetString(GL_VERSION));
	fprintf(log, "extensions: %s\n", glGetString(GL_EXTENSIONS));
	fclose(log);
*/

	EGLint attribList[] = {
		EGL_BUFFER_SIZE, 16, EGL_DEPTH_SIZE, 16,
		EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
		EGL_NONE
	};

	EGLint numConfig;
	if (eglChooseConfig(display, attribList, &config, 1, &numConfig) == EGL_FALSE) {
		return 1;
	}
	if (numConfig < 1) {
		return 1;
	}
	surface = eglCreateWindowSurface(display, config, wnd, NULL);
	if (!surface) {
		return 1;
	}
#ifdef EGL_VERSION_1_2
	eglBindAPI(EGL_OPENGL_ES_API);
#endif
	context = eglCreateContext(display, config, NULL, NULL);
	if (!context) {
		return 1;
	}

	if (eglMakeCurrent(display, surface, surface, context) == EGL_FALSE) {
		return 1;
	}

	RECT rect;
	GetClientRect(wnd, &rect);
	curWidth = rect.right - rect.left; 
	curHeight = rect.bottom - rect.top;

	Knot* k = new Knot(curWidth, curHeight);
	knot = k;
	k->setUpdateVertices(vertexUpdate);
	k->setTexMode(texMode);


//	DWORD start = GetTickCount();
	LARGE_INTEGER startCounter;
	LARGE_INTEGER counterFreq;
	QueryPerformanceFrequency(&counterFreq);
	QueryPerformanceCounter(&startCounter);
	while (true) {
		MSG msg;
		BOOL gotMsg;

		if (active)
			gotMsg = PeekMessage(&msg, NULL, 0, 0, TRUE);
		else {
			gotMsg = GetMessage(&msg, NULL, 0, 0);
			if (!gotMsg)
				break;
		}

		if (gotMsg) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			if (msg.message == WM_QUIT)
				break;
		} else if (active) {
//			DWORD time = GetTickCount() - start;
			LARGE_INTEGER counter;
			QueryPerformanceCounter(&counter);
			int time = (int) ((counter.QuadPart - startCounter.QuadPart) * 1000.0f / counterFreq.QuadPart);
			lastTime = time;
			k->draw(time);
			eglSwapBuffers(display, surface);
		}
	}

	delete k;


	eglMakeCurrent(display, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	eglDestroySurface(display, surface);
	eglDestroyContext(display, context);
	eglTerminate(display);
#ifdef EGL_VERSION_1_2
	eglReleaseThread();
#endif


	return 0;
}
